@@ -248,12 +248,16 @@ app.controller('FMBaseController', function($cacheFactory, $filter, $http, $inje
                     return;
                 }
                 if ($scope.appConfig.ssoProtocol == "SAML") {
+                    let redirectTo = "/";
                     if ($location.path()) {
                         Logger.info("Setting a post-SSO redirect to", $location.path());
                         localStorageService.set("postSSOLoginRedirect", $location.path());
-                        localStorageService.set("postSSOLoginSearch", $httpParamSerializer($location.search()));
+                        const queryParams = $httpParamSerializer($location.search());
+                        localStorageService.set("postSSOLoginSearch", queryParams);
+                        redirectTo = $location.path();
+                        if (queryParams) redirectTo += '?' + queryParams;
                     }
-                    FMAPI.auth.getSAMLRedirectURL(tenantId).success(function(data){
+                    FMAPI.auth.getSAMLRedirectURL(tenantId, redirectTo).success(function(data){
                         window.location = data.url;
                     }).error(function(error) {
                         Logger.info("An error happened during the authentication.");
@@ -291,7 +295,12 @@ app.controller('FMBaseController', function($cacheFactory, $filter, $http, $inje
                 const url = new URL(window.location.href);
                 url.search = search;
                 url.pathname = redirectTo;  // Only follow redirects to a local path, not to another site
-                window.location = url.href;
+                if (window.location.href !== url.href) {
+                    Logger.info("redirect from", window.location.href, "to", url.href);
+                    window.location = url.href;
+                } else {
+                    Logger.info("no redirect needed");
+                }
                 return;
             }
         }
