@@ -509,6 +509,7 @@ app.component('codeEnvUsage', {
             REPORT: (objectId, projectKey) => StateUtils.href.report(objectId, projectKey, {tab: 'edit'}),
             RETRIEVABLE_KNOWLEDGE: (objectId, projectKey) => StateUtils.href.retrievableKnowledge(objectId, projectKey, {tab: 'settings'}),
             API_SERVICE_ENDPOINT: StateUtils.href.lambdaService,
+            AGENT_TOOL: (objectId, projectKey) => StateUtils.href.agentTool(objectId, projectKey, {tab: 'design'}),
             SAVED_MODEL: (objectId, projectKey) => {
                 try {
                     const obj = FullModelLikeIdUtils.parse(objectId);
@@ -586,7 +587,7 @@ app.component('codeEnvUsage', {
             $ctrl.filterUsages();
         };
 
-        function makeOptions(items, field){
+        function makeOptions(items, field, idToNameFunc){
             //dedup item on fields, and generate a label with $itemCount
             return items
                 .reduce((dedup, item) => {
@@ -602,7 +603,13 @@ app.component('codeEnvUsage', {
                     return dedup;
                 }, [])
                 .map(dedup => {
-                    dedup.label = `${dedup.id} (${dedup.count})`;
+                    let name;
+                    if (idToNameFunc) {
+                        name = idToNameFunc(dedup.id);
+                    } else {
+                        name = dedup.id;
+                    }
+                    dedup.label = `${name} (${dedup.count})`;
                     return dedup
                 });
         }
@@ -628,7 +635,7 @@ app.component('codeEnvUsage', {
                             }
                             return usage;
                         });
-                    $ctrl.uiState.options.types = makeOptions($ctrl.accessibleUsages, 'envUsage');
+                    $ctrl.uiState.options.types = makeOptions($ctrl.accessibleUsages, 'envUsage', $filter('typeToName'));
                     $ctrl.uiState.options.projects = makeOptions($ctrl.accessibleUsages, 'projectLabel');
                     $ctrl.filterUsages();
                 })
