@@ -153,7 +153,7 @@ app.controller("ReportsCommonController", function($scope, $rootScope, $state, $
 });
 
 
-app.controller("ReportsListController", function($scope, $rootScope, $controller, $stateParams, DataikuAPI, CreateModalFromTemplate, Dialogs,$state,$q, TopNav, Fn, $filter) {
+app.controller("ReportsListController", function($scope, $rootScope, $controller, $stateParams, DataikuAPI, CreateModalFromTemplate, Dialogs,$state,$q, TopNav, Fn, $filter, PluginConfigUtils) {
     $controller('_TaggableObjectsListPageCommon', {$scope: $scope});
     $controller("ReportsCommonController", {$scope: $scope});
 
@@ -202,13 +202,9 @@ app.controller("ReportsListController", function($scope, $rootScope, $controller
             name: reportType.toLowerCase() + ' report'
         };
 
-        DataikuAPI.reports.listTemplates(reportType).success(function(data) {
-            $scope.availableTemplates = data.templates.filter(function (template) {
-                var plugin = Array.dkuFindFn($rootScope.appConfig.loadedPlugins, function (n) {
-                    return n.id == template.pluginId;
-                });
-                return plugin == null || !plugin.hideComponents;
-            });
+        DataikuAPI.reports.listTemplates(reportType).success(function (data) {
+            const visibilityFilter = PluginConfigUtils.shouldComponentBeVisible($rootScope.appConfig.loadedPlugins);
+            $scope.availableTemplates = data.templates.filter(visibilityFilter);
             $scope.report.template = $scope.availableTemplates[0];
         }).error(setErrorInScope.bind($scope));
 
