@@ -750,13 +750,17 @@
             });
         }
 
-        $rootScope.$on('chartSamplingChanged', function(event, opts) {
+        const unregisterChartSamplingChangedEvent = $rootScope.$on('chartSamplingChanged', function(event, opts) {
             if (angular.equals($scope.chart, opts.chart)) {
                 $scope.chart.summary = null;
                 fetchSummaryAndExecute(true);
             }
         });
 
+        $scope.$on('$destroy', () => {
+            unregisterChartSamplingChangedEvent();
+        });
+
         // chartHandler options
         $scope.noThumbnail = true;
         $scope.addCustomMeasuresToScopeAndCache($scope.insight.params.customMeasures);
