@@ -164,62 +164,93 @@
         };
     });
 
-    app.controller('SavedModelReportInsightController', function($scope, $controller, $stateParams, DataikuAPI, $state, MLRoutingService) {
-        const routePrefix = 'projects.project.dashboards.insights.insight.view.savedmodels';
+    app.controller('SavedModelReportInsightController', function($scope, $controller, $stateParams, DataikuAPI, $state, MLModelsUIRouterStates, GoToStateNameSuffixIfBase, ComputeTransitionStateName) {
+        const baseStateName = 'insight.view.savedmodels';
 
         $controller('SavedModelReportViewCommon', { $scope:$scope });
 
+
         const getSummarySuffix = () => {
             if ($scope.modelData.coreParams.taskType === 'PREDICTION') {
-                return MLRoutingService.getPredictionReportSummaryTab($scope.isDeepHubPrediction(), false);
+                return MLModelsUIRouterStates.getPredictionReportSummaryTab($scope.isDeepHubPrediction(), false);
             } else {
                 return 'summary';
             }
         };
 
-        const goToSummary = () => {
-            goToTab(getSummarySuffix(), null);
-        };
-
         const getRouteType = () => {
             return $scope.modelData.coreParams.taskType === 'PREDICTION' ? 'prediction' : 'clustering';
         };
-        const goToTab = (tab, params) => {
-            $state.go(`${routePrefix}.${getRouteType()}.${tab}`, params, { location: 'replace' });
+
+        const updateUIRouterStateWithTarget = ({ stateName, stateParams }) => {
+            const transitionStateName = ComputeTransitionStateName(baseStateName, stateName);
+
+            if (transitionStateName || !angular.equals($state.current.params, stateParams)) {
+                $state.go(transitionStateName, stateParams, { location: 'replace' });
+            }
         };
 
+        const getUIRouterStateNameTarget = () => {
+            if (!$state.current.name.includes(baseStateName) || !$scope.modelData) {
+                return baseStateName;
+            }
+
+            // if stateName is 'insight.view.savedmodels' redirection on 'insight.view.savedmodels.summary'
+            if ($state.current.name.endsWith(baseStateName)) {
+                return `${baseStateName}.${getRouteType()}.${getSummarySuffix()}`;
+            }
+
+            return $state.current.name.slice($state.current.name.lastIndexOf(baseStateName));
+        };
+
+        const getUIRouterStateNameAndParamsTargetFromTile = (tile) => {
+            const stateNameTypePart = getRouteType();
+            let stateParams = null;
+            let stateNameTabPart;
+
+            if (tile.tileParams.displayMode === 'skins') {
+
+                if (tile.tileParams.advancedOptions && tile.tileParams.advancedOptions.customViews) {
+                    stateNameTabPart = 'modelview';
+                    stateParams = { skinId: tile.tileParams.advancedOptions.customViews.viewId };
+                } else {
+                    stateNameTabPart = getSummarySuffix();
+                }
+
+            } else {
+                const pane = MLModelsUIRouterStates.dashboardTileToSavedModelPane(tile, $scope.modelData.coreParams.taskType === 'CLUSTERING', $scope);
+                stateNameTabPart = pane.route;
+                stateParams = pane.routeParams;
+            }
+
+            return {
+                stateName: `${baseStateName}.${stateNameTypePart}.${stateNameTabPart}`,
+                stateParams: stateParams
+            };
+        };
+
+
         $scope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams) {
             if (!$scope.modelData) {
                 return;
             }
-            MLRoutingService.goToRouteSuffixIfBase($state, toState, toParams, fromState, fromParams, 'projects.project.dashboards.insights.insight.view', `savedmodels.${getRouteType()}.${getSummarySuffix()}`, event);
+            GoToStateNameSuffixIfBase($state, toState, toParams, fromState, fromParams, '.insight.view', `savedmodels.${getRouteType()}.${getSummarySuffix()}`, event);
         });
 
         $scope.getModel(setErrorInScope.bind($scope));
+
         $scope.$watch('modelData', function(nv, ov) {
             if (!nv) {
                 return;
             }
-            if ($state.current.name === routePrefix) {
-                goToSummary();
+            if ($state.current.name.endsWith(baseStateName)) {
+                updateUIRouterStateWithTarget({ stateName: getUIRouterStateNameTarget() });
             }
 
             if ($stateParams.originDashboard) {
                 DataikuAPI.dashboards.getFullInfo($stateParams.projectKey, $stateParams.originDashboard.id).success(function(data) {
                     const tile = data.dashboard.pages.find(page => page.id === $stateParams.originDashboard.pageId).grid.tiles.find(tile => tile.insightId === $stateParams.insightId);
-                    if (tile.tileParams.displayMode === 'skins') {
-                        if (tile.tileParams.advancedOptions && tile.tileParams.advancedOptions.customViews) {
-                            goToTab('modelview', { skinId: tile.tileParams.advancedOptions.customViews.viewId });
-                        } else {
-                            const split = $state.current.name.split('.');
-                            if ($state.current.name !== routePrefix && !split[split.length - 1].endsWith('summary')) { // do not go to summary if we're already on it
-                                goToSummary();
-                            }
-                        }
-                    } else {
-                        const pane = MLRoutingService.dashboardTileToSavedModelPane(tile, $scope.modelData.coreParams.taskType === 'CLUSTERING', $scope);
-                        goToTab(pane.route, pane.routeParams);
-                    }
+                    updateUIRouterStateWithTarget(getUIRouterStateNameAndParamsTargetFromTile(tile));
                 });
             }
         });
@@ -234,7 +265,7 @@
         return {
             scope: true,
             link: function($scope, element, attrs) {
-                if ($state.current.name === 'projects.project.dashboards.insights.insight.view') {
+                if ($state.current.name.endsWith('.insight.view')) {
                     $state.go('.savedmodels', null, { location: 'replace' });
                 }
             }
