@@ -11,6 +11,9 @@
             scope: true,
             controller: function ($scope, $stateParams) {
                 ChartSetErrorInScope.defineInScope($scope);
+                
+                let unregisterChartSamplingChangedEvent;
+                
                 $scope.onLoad = function (projectKey, datasetName, contextProjectKey, datasetSmartName) {
                     //For datasetErrorCTA directive (CTA in case of error while loading dataset sample)
                     $scope.errorCTA = {};
@@ -61,7 +64,11 @@
                         }
                     };
 
-                    $rootScope.$on("chartSamplingChanged", function (event, opts) {
+                    if (unregisterChartSamplingChangedEvent != null) {
+                        unregisterChartSamplingChangedEvent();
+                    }
+
+                    unregisterChartSamplingChangedEvent = $rootScope.$on('chartSamplingChanged', function (event, opts) {
                         if (angular.equals($scope.charts[$scope.currentChart.index], opts.chart)) {
                             $scope.clearCachedSummaries();
                             $scope.fetchColumnsSummaryForCurrentChart().then(function () {
@@ -192,7 +199,12 @@
                         DSSVisualizationThemeUtils.showThemeAppliedSnackbar(currentChart, currentChartCopy);
                     };
 
-                    $scope.$on('$destroy', () => DSSVisualizationThemeUtils.hideThemeAppliedSnackbar());
+                    $scope.$on('$destroy', () => {
+                        DSSVisualizationThemeUtils.hideThemeAppliedSnackbar()
+                        if (unregisterChartSamplingChangedEvent) {
+                            unregisterChartSamplingChangedEvent();
+                        }
+                    });
 
                     $scope.createAndPinInsight = function () {
                         const insights = $scope.charts.map(chart => {
@@ -426,7 +438,7 @@
                         $scope.chart.refreshableSelection._refreshTrigger = new Date().getTime();
                     }
 
-                    $rootScope.$emit("chartSamplingChanged", { chart: $scope.chart });
+                    $rootScope.$emit('chartSamplingChanged', { chart: $scope.chart });
 
                     if (useApplyCancelButtons) {
                         $scope.initialChart = angular.copy($scope.chart); // new reference
@@ -435,7 +447,7 @@
                 };
 
                 $scope.saveNoRefresh = function () {
-                    $rootScope.$emit("chartSamplingChanged", { chart: $scope.chart });
+                    $rootScope.$emit('chartSamplingChanged', { chart: $scope.chart });
                 };
 
                 if (useApplyCancelButtons) {
