@@ -43,7 +43,7 @@
                     items: refs.filter(function(ref) {return !!ref;}).map(function(ref) { return {ref}; }),
                 }
             };
-        };
+        }
         // Creates the recipe object and sends it to the backend
         $scope.doCreateRecipe = function() {
             // make sure recipe is up to date with latest names:
@@ -92,11 +92,9 @@
                 return false;
             }
 
-            if(creationSettings.vectorStoreType === 'PINECONE' && !creationSettings.pineconeIndexName){
-                return false;
-            }
+            return !(creationSettings.vectorStoreType === 'PINECONE' && !creationSettings.pineconeIndexName);
+
 
-            return true;
         };
 
         DataikuAPI.pretrainedModels.listAvailableLLMs($stateParams.projectKey, "TEXT_EMBEDDING_EXTRACTION").success(function(data){
@@ -130,12 +128,12 @@
     });
 
     app.controller("EmbedDocumentsRecipeController", function($scope, $state, $rootScope, $q, $controller, $stateParams, DataikuAPI, RequestCenterService, EmbeddingUtils, 
-    DOCUMENT_SPLITTING_METHOD_MAP, EMBED_DOCS_VECTOR_STORE_UPDATE_METHOD_MAP, VECTOR_STORE_TYPE_MAP, DatasetUtils, RecipesUtils, SmartId) {
+    DOCUMENT_SPLITTING_METHOD_MAP, EMBED_DOCS_VECTOR_STORE_UPDATE_METHOD_MAP, VECTOR_STORE_TYPE_MAP, DatasetUtils, RecipesUtils, SmartId, WT1, ATSurveyService) {
         $controller("RAGEmbeddingRecipeEditionController", {$scope: $scope, VECTOR_STORE_UPDATE_METHOD_MAP: EMBED_DOCS_VECTOR_STORE_UPDATE_METHOD_MAP});
         $controller("_RecipeWithEngineBehavior", {$scope:$scope}); //Controller inheritance // for step status update
 
         DataikuAPI.flow.recipes.nlp.getDefaultEmbedDocumentRecipeSettings($stateParams.projectKey).success(function(data) {
-            if (data && data.defaultVLMId) { 
+            if (data && data.defaultVLMId) {
                 $scope.adminDefaultVLMId = data.defaultVLMId; // admin has defined a default VLM to use on the instance
                 $scope.setDefaultVLM();
             }
@@ -163,7 +161,7 @@
         ];
 
         $scope.VLM_EXTRACTION_PROMPTS = {
-            'SUMMARY_PROMPT': "Generate a concise summary, up to 500 words, derived from the screenshot(s) of document page(s) provided. \n"
+            'SUMMARY_PROMPT': "Generate a concise summary, up to __DKU_NUMBER_OF_CHARS__ characters, derived from the screenshot(s) of document page(s) provided. \n"
                 + "Begin with a brief overview and highlight crucial words, facts, or concepts to enhance both semantic and keyword searchability.\n"
                 + "Omit any references to the original source.\n",
 
@@ -177,7 +175,10 @@
 
         $scope.getActiveVLMfromLLMid = function(vlmId){
             return $scope.availableVLMs.find(llm => llm.id === vlmId);
+        }
 
+        $scope.getPrompt = function(chunkSizeCharacters, promptVersion) {
+            return $scope.VLM_EXTRACTION_PROMPTS[promptVersion].replace("__DKU_NUMBER_OF_CHARS__", chunkSizeCharacters);
         }
 
         DatasetUtils.listDatasetsUsabilityForAny($stateParams.projectKey).success(data => {
@@ -187,14 +188,21 @@
 
         $scope.recipe.params.rules = $scope.recipe.params.rules || [];  // shouldn't be needed as default rules are retrieved from the backend
 
-        $scope.makeDefaultVLMSettings = function(){
+        $scope.makeDefaultVLMSettings = function(rule){
+            let chunkSize = $scope.defaultChunkSize
+            // If we set the chunk size for a structure extraction rule, and then transform
+            // it into a VLM rule, we want to keep the chunk size reflected in the summary
+            // prompt.
+            if (rule?.splittingSettings.chunkSizeCharacters) {
+                chunkSize = rule.splittingSettings.chunkSizeCharacters;
+            }
             return {
                 "llmId" : $scope.recipe.params.defaultVlmId === "DSS_NO_SELECTION"? undefined: $scope.recipe.params.defaultVlmId,
                 "splitUnit" : "PAGE",
                 "customNbPages": 1,
                 "customPagesOverlap": 0,
                 "useCustomPrompt" : false,
-                "customPrompt": $scope.VLM_EXTRACTION_PROMPTS["SUMMARY_PROMPT"],
+                "customPrompt": $scope.getPrompt(chunkSize, "SUMMARY_PROMPT"),
             }
         }
         $scope.makeDefaultStructuredSettings = function(){
@@ -241,7 +249,7 @@
             var ruleToBeUpdated = idx == -1 ? $scope.recipe.params.allOtherRule: $scope.recipe.params.rules[idx];
 
             if (ruleToBeUpdated.actionToPerform == 'VLM'){
-                ruleToBeUpdated.vlmSettings = ruleToBeUpdated.vlmSettings || $scope.makeDefaultVLMSettings();
+                ruleToBeUpdated.vlmSettings = ruleToBeUpdated.vlmSettings || $scope.makeDefaultVLMSettings(ruleToBeUpdated);
                 ruleToBeUpdated.storeInMultimodalColumn = "IMAGES";
             }
             else if (ruleToBeUpdated.actionToPerform == 'STRUCTURED'){
@@ -276,10 +284,17 @@
             }
         }();
 
-         /******  recipe related *****/
-         $scope.hooks.getPayloadData = function () {
+        /******  recipe related *****/
+        $scope.hooks.getPayloadData = function () {
             return angular.toJson($scope.desc);
         };
+        const recipePayloadSave = $scope.hooks.save;
+        $scope.hooks.save = function () {
+            return recipePayloadSave().then(() => {
+                WT1.event("embed-documents-save");
+                ATSurveyService.updateCounter('EmbedDocumentsSave');
+            })
+        }
 
         $scope.hooks.updateRecipeStatus = function() {
             var deferred = $q.defer();
@@ -299,7 +314,7 @@
         };
 
         $scope.showWarningOnDocumentExtractionCodeEnv = () => {
-            if ($scope.recipeStatus && $scope.recipeStatus.strategy) {
+            if ($scope.recipeStatus?.strategy) {
                 return $scope.recipeStatus.strategy.messages?.some(
                     msg => msg.code === "WARN_RECIPE_CODE_ENV_NOT_AVAILABLE"
                 );
@@ -376,7 +391,7 @@
         }
 
         $scope.clickedOnColumnName = function(colName) {
-            var selectedColumn = $scope.metadataDatasetColumns.filter(col => col.name == colName)[0];
+            const selectedColumn = $scope.metadataDatasetColumns.filter(col => col.name == colName)[0];
             selectedColumn.selected = !selectedColumn.selected;
 
             $scope.onSingleColumnSelectionChange();
@@ -404,7 +419,7 @@
         };
         $scope.ruleSchema = undefined;
         $scope.updateRulesSchema = function(){
-            var ruleSchema = angular.copy(baseRuleSchema);
+            let ruleSchema = angular.copy(baseRuleSchema);
             if($scope.metadataDatasetColumns && $scope.metadataDatasetColumns.length > 0){
                 $scope.metadataDatasetColumns.forEach(function(col) {
                     ruleSchema.columns.push({
@@ -487,13 +502,12 @@
             return $scope.recipe.inputs['metadata_dataset'].items.length > 0;
         }
 
-        $scope.overrideCustomPromptWith = function (vlmSettings, promptVersion) {
-
-            if(!promptVersion || !$scope.VLM_EXTRACTION_PROMPTS[promptVersion]){
-                return;
+        $scope.overrideCustomPromptWith = function (rule, promptVersion) {
+            if (promptVersion && $scope.VLM_EXTRACTION_PROMPTS[promptVersion]) {
+                rule.vlmSettings.customPrompt = $scope.getPrompt(rule.splittingSettings.chunkSizeCharacters, promptVersion);
             }
-            vlmSettings.customPrompt = $scope.VLM_EXTRACTION_PROMPTS[promptVersion];
         }
+
     });
  
 
