@@ -33,14 +33,12 @@ class AbstractTimeseriesSplitHandler(object):
         self.prediction_length = core_params[PREDICTION_LENGTH]
         self.test_size = core_params[EVALUATION_PARAMS][TEST_SIZE]
 
-        self._time_steps_ranks = None
-
     def prepare_split_dataframe(self, df, required_timeseries_training_size, skip_too_short_timeseries=False):
         """
         Prepare dataframe for splitting (checking time series sizes and potentially skipping
         those that are too short)
 
-        :param df: Input dataframe to be split. /!\ HAS TO BE SORTED BY TIME /!\
+        :param df: Input dataframe to be split. /! HAS TO BE SORTED BY TIME /!
         :type df: pd.DataFrame
         :param required_timeseries_training_size: Minimum required size for the train set of the smallest split
                for any time series.
@@ -82,14 +80,23 @@ class AbstractTimeseriesSplitHandler(object):
 
         return df
 
-    def _retrieve_short_timeseries_identifiers_with_reasons(self, df, min_required_size):
+    def get_min_size_required_for_split(self, required_timeseries_training_size=None):
+        """
+        Given an algorithm required size for training, return the minimal required size for split
+        :param required_timeseries_training_size: optional minimum required size.
+        :rtype required_timeseries_training_size: Optional[int]
+        :return: int
+        """
+        raise NotImplementedError
+
+    def retrieve_short_timeseries_identifiers_with_reasons(self, df, min_required_size):
         """
         Iterate over the dataframe timeseries and gather the valid and valid identifiers.
         :param df: the full timeseries dataframe
         :type df: pd.DataFrame
         :param min_required_size: optional minimum required size.
         :rtype min_required_size: Optional[int]
-        :return: A tuple containing the list of valid and invalid (too short) identifiers. For invalid indentifiers it contains a tuple with the identifier name, the type of the invalid ds (train/test) and the required size.
+        :return: A tuple containing the list of valid and invalid (too short) identifiers. For invalid identifiers it contains a tuple with the identifier name, the type of the invalid ds (train/test) and the required size.
         :rtype: Tuple[list[str], list[Tuple[str, str, int]]
         """
         raise NotImplementedError
@@ -101,7 +108,7 @@ class AbstractTimeseriesSplitHandler(object):
         """
         raise NotImplementedError
 
-    def filter_timeseries(self, df, required_timeseries_size = None, skip_too_short_timeseries=False):
+    def filter_timeseries(self, df, required_timeseries_size=None, skip_too_short_timeseries=False):
         """
         Filters the full input dataframe with potential too short timeseries if `skip_too_short_timeseries` == `True`. Otherwise, it raises an exception.
         :param df: the full input dataframe
@@ -113,7 +120,7 @@ class AbstractTimeseriesSplitHandler(object):
         :return: the filtered dataframe
         :rtype: pd.DataFrame
         """
-        kept_timeseries_identifier_values, too_short_timeseries_identifiers_with_reasons = self._retrieve_short_timeseries_identifiers_with_reasons(df, required_timeseries_size)
+        kept_timeseries_identifier_values, too_short_timeseries_identifiers_with_reasons = self.retrieve_short_timeseries_identifiers_with_reasons(df, required_timeseries_size)
         if skip_too_short_timeseries:
             df = self._skip_too_short_timeseries(df, required_timeseries_size, too_short_timeseries_identifiers_with_reasons, kept_timeseries_identifier_values)
         elif len(too_short_timeseries_identifiers_with_reasons):
@@ -134,7 +141,7 @@ class AbstractTimeseriesSplitHandler(object):
         """
         error_message = "The following time series are too short for training: "
         for (identifier, dataset, length) in too_short_timeseries_identifiers_with_reasons:
-            error_message += "{}: {} length: {} < Min length: {}; ".format(pretty_timeseries_identifiers(identifier) if self.timeseries_identifier_columns else identifier, dataset, length, self.prediction_length if dataset == "test dataset" else required_training_size)
+            error_message += "{}: {} length: {} < Min length: {}; ".format(pretty_timeseries_identifiers(identifier) if self.timeseries_identifier_columns else identifier, dataset, length, self.prediction_length if "test dataset" in dataset else required_training_size)
         return error_message
 
     def split(self, df, split_id=None):
@@ -161,23 +168,6 @@ class AbstractTimeseriesSplitHandler(object):
         """
         raise NotImplementedError
 
-    def compute_time_steps_ranks(self, df):
-        """Compute time-steps ranks
-        - time_steps_ranks is a pd.Series of the rank of each time steps relatively to its time series.
-        - time steps are ranked by reverse chronological order (from most recent within each time series (0) to oldest).
-        - time_steps_ranks itself is zero-indexed, with each 'value' corresponding to index of the df.
-        :param df: a dataframe
-        :type pd.Dataframe
-        :return: a series of rank
-        :rtype: pd.Series
-        """
-        #
-        if self.timeseries_identifier_columns:
-            df_grouped_by_timeseries = df.groupby(_groupby_compat(self.timeseries_identifier_columns))
-            self._time_steps_ranks = df_grouped_by_timeseries[self.time_column].rank(method="first", ascending=False) - 1
-        else:
-            self._time_steps_ranks = df[self.time_column].rank(method="first", ascending=False) - 1
-
 class CustomTrainTestTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
     """Creates splits based on a list of date intervals.
     Attributes:
@@ -187,76 +177,15 @@ class CustomTrainTestTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
         super(CustomTrainTestTimeseriesSplitHandler, self).__init__(core_params)
         intervals = core_params.get(CUSTOM_TRAIN_TEST_INTERVALS, [])
         self.timestep_params = core_params["timestepParams"]
-        if len(intervals) > 1:
-            raise ValueError("Several intervals were specified. Interval based splitting only support one interval")
         # Intervals should be a list[Dict[str, Tuple[str, str]]) for train/test intervals
         self.intervals = [{ "train": [np.datetime64(interval["train"][0]), np.datetime64(interval["train"][1])], "test": [np.datetime64(interval["test"][0]), np.datetime64(interval["test"][1])] } for interval in intervals]
-        self._round_interval_boundaries()
-
-    def _round_interval_boundaries(self):
-        """
-        Expands user-defined intervals to align with the time series' frequency,
-        ensuring splits don't cut across time periods (e.g., a month).
-
-        The method expands the interval to cover the full duration of the periods
-        containing the user-provided start and end dates. For example, for monthly
-        data, '2020-03-15' to '2020-05-10' becomes '2020-03-01' to '2020-05-31'.
-
-        As a result, the final interval can be substantially larger and differ from
-        the one specified by the user.
-        """
-        timeunit = self.timestep_params.get("timeunit")
-        if not timeunit:
-            return
-        for interval in self.intervals:
-            for part in ["train", "test"]:
-                start_date = pd.Timestamp(interval[part][0])
-                end_date = pd.Timestamp(interval[part][1])
-
-                new_start, new_end = start_date, end_date
-                if timeunit == "WEEK":
-                    java_to_python_days = {
-                        1: 6,
-                        2: 0,
-                        3: 1,
-                        4: 2,
-                        5: 3,
-                        6: 4,
-                        7: 5
-                    }
-                    end_of_week_day_java = self.timestep_params.get("endOfWeekDay", 1)
-                    end_of_week_day = java_to_python_days[end_of_week_day_java]
-                    new_start = pd.tseries.offsets.Week(weekday=end_of_week_day).rollback(start_date).normalize()
-                    new_end = pd.tseries.offsets.Week(weekday=end_of_week_day).rollforward(end_date).normalize() + pd.Timedelta(days=1) - pd.Timedelta(milliseconds=1)
-                elif timeunit in ["MONTH", "QUARTER", "HALF_YEAR"]:
-                    new_start = pd.tseries.offsets.MonthBegin().rollback(start_date).normalize()
-                    new_end = pd.tseries.offsets.MonthBegin().rollback(end_date).normalize() + pd.DateOffset(months=1) - pd.Timedelta(milliseconds=1)
-                elif timeunit == "YEAR":
-                    new_start = pd.tseries.offsets.YearBegin().rollback(start_date).normalize()
-                    new_end = pd.tseries.offsets.YearBegin().rollback(end_date).normalize() + pd.DateOffset(years=1) - pd.Timedelta(milliseconds=1)
-                else:
-                    # For smaller time units, we floor the start date and ceil the end date
-                    freq_map = {"SECOND": "s", "MINUTE": "min", "HOUR": "h", "DAY": "D", "BUSINESS_DAY": "D"}
-                    timeunit_to_offset = {
-                        "BUSINESS_DAY": pd.DateOffset(days=1),
-                        "DAY": pd.DateOffset(days=1),
-                        "HOUR": pd.DateOffset(hours=1),
-                        "MINUTE": pd.DateOffset(minutes=1),
-                        "SECOND": pd.DateOffset(seconds=1),
-                    }
-
-                    if timeunit in freq_map:
-                        freq = freq_map[timeunit]
-                        new_start = start_date.floor(freq)
-                        new_end = end_date.floor(freq) + timeunit_to_offset[timeunit] - pd.Timedelta(milliseconds=1)
-                interval[part] = [np.datetime64(new_start), np.datetime64(new_end)]
 
     def _single_split(self, interval_id, df):
         interval = self.intervals[interval_id]
         fold_train_start_date, fold_train_end_date = interval["train"][0], interval["train"][1]
         fold_test_start_date, fold_test_end_date = interval["test"][0], interval["test"][1]
-        train_indices = (df[self.time_column] >= fold_train_start_date) & (df[self.time_column] <= fold_train_end_date)
-        test_indices = (df[self.time_column] >= fold_test_start_date) & (df[self.time_column] <= fold_test_end_date)
+        train_indices = (df[self.time_column] >= fold_train_start_date) & (df[self.time_column] < fold_train_end_date)
+        test_indices = (df[self.time_column] >= fold_test_start_date) & (df[self.time_column] < fold_test_end_date)
         historical_indices = (df[self.time_column] >= fold_train_start_date) & (df[self.time_column] < fold_test_start_date)
 
         train_df = df.loc[train_indices]
@@ -273,34 +202,50 @@ class CustomTrainTestTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
         else:
             yield self._single_split(split_id, df)
 
+    def get_min_size_required_for_split(self, required_timeseries_training_size=None):
+        return required_timeseries_training_size
+
     def prepare_split_dataframe(self, df, required_timeseries_training_size, skip_too_short_timeseries=False):
-        self.compute_time_steps_ranks(df)
         for interval in self.intervals:
             self._assert_valid_interval(interval)
         return self.filter_timeseries(df,
                                       required_timeseries_size=required_timeseries_training_size,
                                       skip_too_short_timeseries=skip_too_short_timeseries)
 
-    def _retrieve_short_timeseries_identifiers_with_reasons(self, df, min_required_training_size):
+    def retrieve_short_timeseries_identifiers_with_reasons(self, df, min_required_training_size):
         too_short_timeseries_identifiers_with_reasons = []
         kept_timeseries_identifier_values = []
         if self.timeseries_identifier_columns:
             df_grouped_by_timeseries = df.groupby(_groupby_compat(self.timeseries_identifier_columns))
             for identifier_values, timeseries_df in df_grouped_by_timeseries:
-                train_df, test_df, _ = self._single_split(0, timeseries_df)
-                if len(train_df) < min_required_training_size: # We need to be able to train and to score at least once
-                    too_short_timeseries_identifiers_with_reasons.append((encode_timeseries_identifier(identifier_values, self.timeseries_identifier_columns), "train dataset", len(train_df)))
-                elif len(test_df) < self.prediction_length:
-                    too_short_timeseries_identifiers_with_reasons.append((encode_timeseries_identifier(identifier_values, self.timeseries_identifier_columns), "test dataset", len(test_df)))
-                else:
+                kept_identifier = True
+                folds_number = len(self.intervals) if self.intervals else 1
+                for fold_id in range(folds_number):
+                    train_df, test_df, _ = self._single_split(fold_id, timeseries_df)
+                    if len(train_df) < min_required_training_size:  # We need to be able to train and to score at least once
+                        too_short_timeseries_identifiers_with_reasons.append(
+                            (encode_timeseries_identifier(identifier_values, self.timeseries_identifier_columns), "train dataset of fold " + str(fold_id + 1),
+                             len(train_df)))
+                        kept_identifier = False
+                    elif len(test_df) < self.prediction_length:
+                        too_short_timeseries_identifiers_with_reasons.append(
+                            (encode_timeseries_identifier(identifier_values, self.timeseries_identifier_columns), "test dataset of fold " + str(fold_id + 1),
+                             len(test_df)))
+                        kept_identifier = False
+                if kept_identifier:
                     kept_timeseries_identifier_values.append(identifier_values)
         else:
-            train_df, test_df, _ = self._single_split(0, df)
-            if len(train_df) < min_required_training_size:
-                too_short_timeseries_identifiers_with_reasons.append((SINGLE_TIMESERIES_IDENTIFIER, "train dataset", len(train_df)))
-            elif len(test_df) < self.prediction_length:
-                too_short_timeseries_identifiers_with_reasons.append((SINGLE_TIMESERIES_IDENTIFIER, "test dataset", len(test_df)))
-            else:
+            kept_identifier = True
+            folds_number = len(self.intervals) if self.intervals else 1
+            for fold_id in range(folds_number):
+                train_df, test_df, _ = self._single_split(fold_id, df)
+                if len(train_df) < min_required_training_size:
+                    too_short_timeseries_identifiers_with_reasons.append((SINGLE_TIMESERIES_IDENTIFIER, "train dataset of fold " + str(fold_id + 1), len(train_df)))
+                    kept_identifier = False
+                elif len(test_df) < self.prediction_length:
+                    too_short_timeseries_identifiers_with_reasons.append((SINGLE_TIMESERIES_IDENTIFIER, "test dataset of fold " + str(fold_id + 1), len(test_df)))
+                    kept_identifier = False
+            if kept_identifier:
                 kept_timeseries_identifier_values.append(SINGLE_TIMESERIES_IDENTIFIER)
 
         return kept_timeseries_identifier_values, too_short_timeseries_identifiers_with_reasons
@@ -318,7 +263,7 @@ class CustomTrainTestTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
             raise ValueError("Invalid interval: train start date is after train end date")
         if test_start_date > test_end_date:
             raise ValueError("Invalid interval: test start date is after test end date")
-        if train_end_date >= test_start_date:
+        if test_start_date < train_end_date:
             raise ValueError("Invalid interval: train end date is after test start date")
 
     def _generate_error_message_cta(self):
@@ -390,42 +335,59 @@ class KFoldTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
                 2. train: [2, 3, 4, 5, 6], test: [7, 8], historical: [2, 3, 4, 5, 6]
                 3. train: [4, 5, 6, 7, 8], test: [9, 10], historical: [4, 5, 6, 7, 8]
 
-        :param df: Input dataframe to be split. /!\ HAS TO BE SORTED BY TIME /!\
+        :param df: Input dataframe to be split. /! HAS TO BE SORTED BY TIME /!
         :param split_id: Id of the split to generate (0 is the first split with the smallest train set).
                If not set, all splits are generated.
         """
-        if self._time_steps_ranks is None:
-            logger.warning("Time steps ranks not available. Make sure `prepare_split_dataframe` is called before `split`.")
-            self.compute_time_steps_ranks(df)
+        time_steps_ranks = self.compute_time_steps_ranks(df)
         # time step rank where the test time series ends (inclusive)
         most_recent_test_timestep = self.test_size * (self.n_splits - split_id - 1) * (2 if self.fold_offset else 1)
         # time step rank where the train time series ends (inclusive).
         # It is greater than most_recent_test_timestep as the time steps are ranked by reverse chronological order
         most_recent_train_timestep = most_recent_test_timestep + self.test_size
         # A higher time step rank means an older time step
-        train_indices = self._time_steps_ranks >= most_recent_train_timestep
+        train_indices = time_steps_ranks >= most_recent_train_timestep
 
         # If every split is of equal duration, the train start is offset from the beginning
         if self.equal_duration_folds:
             beginning_offset = self.test_size * split_id * (2 if self.fold_offset else 1)
-            oldest_train_timestamp = self._time_steps_ranks.iloc[0] - beginning_offset
-            train_indices &= self._time_steps_ranks <= oldest_train_timestamp
-        test_indices = (self._time_steps_ranks >= most_recent_test_timestep) & (self._time_steps_ranks < most_recent_train_timestep)
+            oldest_train_timestamp = time_steps_ranks.iloc[0] - beginning_offset
+            train_indices &= time_steps_ranks <= oldest_train_timestamp
+        test_indices = (time_steps_ranks >= most_recent_test_timestep) & (time_steps_ranks < most_recent_train_timestep)
         train_df = df.loc[train_indices]
         test_df = df.loc[test_indices]
         return train_df, test_df, train_df
 
+    def compute_time_steps_ranks(self, df):
+        """Compute time-steps ranks
+        - time_steps_ranks is a pd.Series of the rank of each time steps relatively to its time series.
+        - time steps are ranked by reverse chronological order (from most recent within each time series (0) to oldest).
+        - time_steps_ranks itself is zero-indexed, with each 'value' corresponding to index of the df.
+        :param df: a dataframe
+        :type pd.Dataframe
+        :return: a series of rank
+        :rtype: pd.Series
+        """
+        #
+        if self.timeseries_identifier_columns:
+            df_grouped_by_timeseries = df.groupby(_groupby_compat(self.timeseries_identifier_columns))
+            return df_grouped_by_timeseries[self.time_column].rank(method="first", ascending=False) - 1
+        else:
+            return df[self.time_column].rank(method="first", ascending=False) - 1
+
+    def get_min_size_required_for_split(self, required_timeseries_training_size=None):
+        return self.n_splits * self.test_size + required_timeseries_training_size
+
     def prepare_split_dataframe(self, df, required_timeseries_training_size, skip_too_short_timeseries=False):
         # train set of first window (length = full_length - n_splits * test_size)
         # (if fold_offset: length = full_length - (2 * n_splits - 1) * test_size)
         # must be bigger than the required time series size for training a given algorithm, i.e.:
-        self.compute_time_steps_ranks(df)
-        min_required_length_for_split = self.n_splits * self.test_size + required_timeseries_training_size
+        min_required_length_for_split = self.get_min_size_required_for_split(required_timeseries_training_size)
         if self.fold_offset:
             min_required_length_for_split += (self.n_splits - 1) * self.test_size
         return self.filter_timeseries(df, min_required_length_for_split, skip_too_short_timeseries)
 
-    def _retrieve_short_timeseries_identifiers_with_reasons(self, df, min_required_length_for_split):
+    def retrieve_short_timeseries_identifiers_with_reasons(self, df, min_required_length_for_split):
         if self.timeseries_identifier_columns:
             df_grouped_by_timeseries = df.groupby(_groupby_compat(self.timeseries_identifier_columns))
             timeseries_sizes = df_grouped_by_timeseries.size()
@@ -440,7 +402,7 @@ class KFoldTimeseriesSplitHandler(AbstractTimeseriesSplitHandler):
                 kept_timeseries_identifier_values = set()
             else:
                 too_short_timeseries_identifiers_with_reasons = []
-                kept_timeseries_identifier_values = set(SINGLE_TIMESERIES_IDENTIFIER)
+                kept_timeseries_identifier_values = {SINGLE_TIMESERIES_IDENTIFIER}
 
         return kept_timeseries_identifier_values, too_short_timeseries_identifiers_with_reasons
 
