@@ -231,7 +231,7 @@ class DoclingExtractorPipeline:
                 if ocr_options is None:
                     logger.warning("The provided OCR engine is not available, deactivating OCR for extraction")
                 else:
-                    ocr_options.force_full_page_ocr = True
+                    ocr_options.force_full_page_ocr = False # To apply OCR on each image if pdf is not fully scanned. This will ensure we get good results on embedded images.
             pipeline_options = PdfPipelineOptions(do_table_structure=True, do_ocr=(needs_ocr and ocr_options is not None),
                                                   ocr_options=(ocr_options if ocr_options is not None else EasyOcrOptions()))  # Can't set ocr_options to None
             # Pdf extraction with docling requires those IBM models tableformer and layout:  https://huggingface.co/ds4sd/docling-models
