@@ -248,6 +248,13 @@ dku_exec "df -i"
 
 dku_exec "lsblk"
 dku_exec "lsblk -t"
+dku_exec "blkid"
+dku_exec "mount"
+dku_exec "cat /etc/fstab"
+dku_exec "cat /etc/mtab"
+dku_exec "findmnt -D"
+dku_exec "findmnt -n -o SOURCE /data"
+dku_exec "ls -Rla /dev/disk/"
 
 dku_exec_to_file "dmesg" $DIAG_TMP_DIR/dmesg.txt
 
